/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver
 * File:    Main.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.communication.bus.Queue;
import de.fhhannover.inform.iron.mapserver.communication.bus.messages.ActionSeries;
import de.fhhannover.inform.iron.mapserver.communication.bus.messages.Event;
import de.fhhannover.inform.iron.mapserver.communication.exceptions.ServerInitialException;
import de.fhhannover.inform.iron.mapserver.communication.http.ActionProcessor;
import de.fhhannover.inform.iron.mapserver.communication.http.ChannelAcceptor;
import de.fhhannover.inform.iron.mapserver.communication.http.ChannelRep;
import de.fhhannover.inform.iron.mapserver.communication.ifmap.EventProcessor;
import de.fhhannover.inform.iron.mapserver.communication.ifmap.SessionTimerFactory;
import de.fhhannover.inform.iron.mapserver.datamodel.DataModelService;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.AlreadyObservedException;
import de.fhhannover.inform.iron.mapserver.logging.Logging;
import de.fhhannover.inform.iron.mapserver.provider.BasicAuthProvider;
import de.fhhannover.inform.iron.mapserver.provider.BasicAuthProviderPropImpl;
import de.fhhannover.inform.iron.mapserver.provider.IncremetalSessionIdProviderImpl;
import de.fhhannover.inform.iron.mapserver.provider.ProviderInitializationException;
import de.fhhannover.inform.iron.mapserver.provider.PublisherIdGenerator;
import de.fhhannover.inform.iron.mapserver.provider.PublisherIdGeneratorImpl;
import de.fhhannover.inform.iron.mapserver.provider.PublisherIdProvider;
import de.fhhannover.inform.iron.mapserver.provider.PublisherIdProviderPropImpl;
import de.fhhannover.inform.iron.mapserver.provider.ServerConfigurationProvider;
import de.fhhannover.inform.iron.mapserver.provider.ServerConfigurationProviderPropImpl;
import de.fhhannover.inform.iron.mapserver.provider.SessionIdProvider;
import de.fhhannover.inform.iron.mapserver.utils.RequestTransformer;
import de.fhhannover.inform.iron.mapserver.utils.RequestTransformerFactory;
import de.fhhannover.inform.iron.mapserver.utils.ResponseCreator;
import de.fhhannover.inform.iron.mapserver.utils.ResponseCreatorFactory;

/**
 * Entry point to run the IF-MAP Server implementation.
 * Dependencies are resolved here.
 * 
 * @author aw
 *
 */
public class Main {
	
	
	private static final String MAIN_CONFIGUARTION_FILE = "ifmap.properties";
	
	private static Logger sLogger;
	{
		sLogger = Logging.getTheLogger();
	}
	
	/**
	 * represents global queue for events.
	 */
	private Queue<Event> mEventQueue;
	
	/**
	 * represents the global queue for actions.
	 */
	private Queue<ActionSeries> mActionQueue;
	
	/**
	 * represents the {@link DataModelService} instance to be used.
	 */
	private DataModelService mDataModelService;
	
	/**
	 * represents the {@link ChannelRep} to be used by {@link ActionProcessor}
	 * and {@link ChannelAcceptor}.
	 */
	private ChannelRep mChannelRep;
	
	/**
	 * represents the {@link ServerConfigurationProvider} instance to be used.
	 */
	private ServerConfigurationProvider mServerConfig;
	
	/**
	 * represents the {@link PublisherIdProvider} instance to be used.
	 */
	private PublisherIdProvider mPublisherIdProvider;
	
	/**
	 * represents the {@link SessionIdProvider} instance to be used.
	 */
	private SessionIdProvider mSessionIdProvider;
	
	/**
	 * represents the {@link ActionProcessor} instance which uses the action queue.
	 */
	private ActionProcessor mActionProcessor;
	
	
	/**
	 * represents the {@link EventProcessor} instance which uses the event queue.
	 */
	private EventProcessor mEventProcessor;

	/**
	 * represents the {@link RequestTransformer} to be used by the {@link EventProcessor}.
	 */
	private RequestTransformer mRequestTransformer;
	
	/**
	 * represents the {@link ResponseCreator} to be used by the {@link EventProcessor}.
	 */
	private ResponseCreator mResponseCreator;
	
	
	/**
	 * 
	 */
	private ChannelAcceptor mChannelAcceptor;

	/**
	 * 
	 */
	private BasicAuthProvider mBasicAuthProvider;
	
	/**
	 * 
	 */
	private SessionTimerFactory mSessionTimerFactory;

	/**
	 * represents the instance to be used to generate publisher-id's
	 */
	private PublisherIdGenerator mPublisherIdGenerator;
	
	
	public Main() throws ServerInitialException {
		init();
	}
	
	private void init() throws ServerInitialException {
		
		try {
			mServerConfig = new ServerConfigurationProviderPropImpl(MAIN_CONFIGUARTION_FILE);
			mPublisherIdProvider = new PublisherIdProviderPropImpl(mServerConfig);
			mPublisherIdGenerator = new PublisherIdGeneratorImpl();
			mSessionIdProvider = new IncremetalSessionIdProviderImpl();
			mBasicAuthProvider = new BasicAuthProviderPropImpl(mServerConfig);
		} catch (ProviderInitializationException e) {
			throw new ServerInitialException("A Provider could not be initialized: " +
					e.getMessage());
		}
		
		// processor and queue part
		int eventForwarders = mServerConfig.getEventProcessorForwardersCount();
		int eventWorkers = mServerConfig.getEventProcessorWorkersCount();
		int actionForwarders = mServerConfig.getActionProcessorForwardersCount();
		int actionWorkers = mServerConfig.getActionProcessorWorkersCount();
		mEventQueue = new Queue<Event>();
		mActionQueue = new Queue<ActionSeries>();
		mEventProcessor = new EventProcessor(mEventQueue, eventWorkers, eventForwarders);
		mActionProcessor = new ActionProcessor(mActionQueue, actionWorkers, actionForwarders);
		
		mRequestTransformer = RequestTransformerFactory.newRequestTransformer();
		mResponseCreator = ResponseCreatorFactory.newResponseCreator();

		mDataModelService = DataModelService.newInstance(mServerConfig);
		
		mSessionTimerFactory = new SessionTimerFactory(mEventQueue, mServerConfig);
		
		
		// initialize the event processor...
		mEventProcessor.setActionQueue(mActionQueue);
		mEventProcessor.setPublisherIdProv(mPublisherIdProvider);
		mEventProcessor.setSessionIdProv(mSessionIdProvider);
		mEventProcessor.setRequestTransformer(mRequestTransformer);
		mEventProcessor.setResponseCreator(mResponseCreator);
		mEventProcessor.setServerConfiguration(mServerConfig);
		mEventProcessor.setDataModel(mDataModelService);
		mEventProcessor.setSessionTimerFactory(mSessionTimerFactory);
		mEventProcessor.setPublisherIdGenerator(mPublisherIdGenerator);

		try {
			mDataModelService.registerSubscriptionObserver(mEventProcessor);
		} catch (AlreadyObservedException e) {
			throw new ServerInitialException("Could not register event" +
					" processor as SubscriptionObserver.");
		}
		
		
		mChannelRep = new ChannelRep();
		
		// TOOD: initialize the action processor
		mActionProcessor.setChannelRepository(mChannelRep);
		
		
		mChannelAcceptor = new ChannelAcceptor(mServerConfig,
				mEventQueue, mChannelRep, mBasicAuthProvider);
	}

	public void goForIt() {
		
		sLogger.info("Starting irond MAPServer...");
		mEventProcessor.start();
		mActionProcessor.start();
		mChannelAcceptor.start();
	}

	
	public static void main(String[] args) {
		
		try {
			Main main = new Main();
			main.goForIt();
		} catch (ServerInitialException e) {
			sLogger.error("Could not run the server :'-(");
			sLogger.error(e.getMessage());
		}
	}

}
