/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication.http
 * File:    CertificateAuth.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication.http;

import java.net.Socket;

import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.security.cert.X509Certificate;

import org.apache.http.HttpRequest;

import de.fhhannover.inform.iron.mapserver.communication.ClientIdentifier;

/**
 * Implementation for the certificate-based authentication.
 * 
 * @author tr
 *
 */
public class CertificateAuth implements ChannelAuth {

	private ClientIdentifier mClientId;
	private Socket mCredentialSocket;
	
	public CertificateAuth(Socket credentialSocket){
		mCredentialSocket = credentialSocket;
	}
	
	@Override
	public void authenticate(HttpRequest request) throws AuthException
	{	
		// creating the ClientIdentifier based on the dn-name
		X509Certificate[] x509;
		String dn = null;
		try {
			x509 = ((SSLSocket) mCredentialSocket).getSession().getPeerCertificateChain();
			dn = x509[0].getIssuerDN().getName();
		} catch (SSLPeerUnverifiedException e) {
			throw new AuthException("SSL verification failed!");
		}
		ClientIdentifier newClId = new ClientIdentifier(makeNice(dn));
		
		if (mClientId != null) {
			if (!mClientId.equals(newClId))
				throw new AuthException("Something bad happened");
		} else {
			mClientId = newClId;
		}
	}


	@Override
	public ClientIdentifier getClientIdentifier() {		
		return mClientId;
	}


	private static final String CN_ATTR = "CN=";
	private static final String OU_ATTR = "OU=";
	private static final String O_ATTR = "O=";
	private static final String C_ATTR = "C=";
	
	/**
	 * From a issuer name, create something that looks like a domain name.
	 * It might not always work, but it's much better than using a string
	 * like CN=blub, OU=fh-hannover....
	 * 
	 * @param dn
	 * @return
	 */
	private String makeNice(String dn) {
		StringBuffer ret = new StringBuffer();
		
		int cnIdx = dn.indexOf(CN_ATTR);
		int ouIdx = dn.indexOf(OU_ATTR);
		int oIdx = dn.indexOf(O_ATTR);
		int cIdx = dn.indexOf(C_ATTR);
		
		if (cnIdx > -1) {
			cnIdx += CN_ATTR.length();
			String cn = dn.substring(cnIdx);
			ret.append(cutOffAtFirstCommaAddPoint(cn));
		}

		if (ouIdx > -1) {
			ouIdx += OU_ATTR.length();
			String ou = dn.substring(ouIdx);
			ret.append(cutOffAtFirstCommaAddPoint(ou));
		}
		
		if (oIdx > -1) {
			oIdx += O_ATTR.length();
			String o = dn.substring(oIdx);
			ret.append(cutOffAtFirstCommaAddPoint(o));
		}
		
		if (cIdx > -1) {
			cIdx += C_ATTR.length();
			String c = dn.substring(cIdx);
			ret.append(cutOffAtFirstCommaAddPoint(c));
		}
		
		// remove trailing point
		ret.deleteCharAt(ret.length() - 1);
		return ret.toString().replace(' ', '_').toLowerCase();
	}

	private String cutOffAtFirstCommaAddPoint(String str) {
		int commaIdx = str.indexOf(',');
		if (commaIdx > -1) {
			return str.substring(0, commaIdx) + ".";
			
		}
		return str;
	}
}
