/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication.http
 * File:    ChannelAcceptor.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication.http;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.Socket;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import javax.net.ServerSocketFactory;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.communication.ChannelIdentifier;
import de.fhhannover.inform.iron.mapserver.communication.bus.Queue;
import de.fhhannover.inform.iron.mapserver.communication.bus.messages.Event;
import de.fhhannover.inform.iron.mapserver.communication.exceptions.ServerInitialException;
import de.fhhannover.inform.iron.mapserver.logging.Logging;
import de.fhhannover.inform.iron.mapserver.provider.BasicAuthProvider;
import de.fhhannover.inform.iron.mapserver.provider.ServerConfigurationProvider;
/**
 * Opens sockets for the basic-authentification and certificate-based auth. After a connect from
 * a client, the socket is deligated to a {@link ChannelThread}. 
 *
 * @author tr
 *
 */
public class ChannelAcceptor extends Thread {

	/**
	 * Holds the configuration as specified in <code>ifmap-properties</code>
	 */
	private ServerConfigurationProvider mConfReader;
	
	/**
	 * The SSL socket for basic authentication
	 */
	private SSLServerSocket mBasicAuthSocket;

	/**
	 * The SSL socket for credential authentication
	 */
	private SSLServerSocket mCredentialAuthSocket;

	/**
	 * Counter for thread-channels
	 */
	private int mCounter = 0;

	/**
	 * The two supported authentification methodes
	 */
	private BasicAuth mBasicAuth;
	private CertificateAuth mCredentialAuth;

	/**
	 * Channel threads repository
	 */
	private ChannelRep mChannelRep;
	
	/**
	 * The logger
	 */
	private static Logger logger;

	private Queue<Event> mEventQueue;
	
	private BasicAuthProvider mBasicAuthProvider;
	

	static {
		logger = Logging.getTheLogger();
	}

	
	public ChannelAcceptor(ServerConfigurationProvider serverConf,
			Queue<Event> eventQueue, ChannelRep channelRep,
			BasicAuthProvider basicAuthProvider) throws ServerInitialException {
		
		mConfReader = serverConf;
		mEventQueue = eventQueue;
		mChannelRep = channelRep;
		mBasicAuthProvider = basicAuthProvider;
		
		logger.info("Using keystore path: " + mConfReader.getKeyStoreFileName());

		System.setProperty("javax.net.ssl.keyStore", mConfReader.getKeyStoreFileName());
		System.setProperty("javax.net.ssl.keyStorePassword", mConfReader.getKeyStorePasswort());
		
		ServerSocketFactory ssocketFactory = SSLServerSocketFactory.getDefault();
		
		
		int basicAuthPort = mConfReader.getBasicAuthPort();

		if (basicAuthPort != 0) {			
			try {
				mBasicAuthSocket = (SSLServerSocket) ssocketFactory.createServerSocket(basicAuthPort);
			} catch (IOException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			}
		}

		int credentialAuthPort = mConfReader.getCertAuthPort();
		
		if (credentialAuthPort != 0) {
			System.setProperty("javax.net.ssl.trustStore", mConfReader.getTrustStoreFileName());					
			System.setProperty("javax.net.ssl.trustStorePassword", 
					mConfReader.getTrustStorePasswort());					
			
			try {
				SecureRandom random = SecureRandom.getInstance("SHA1PRNG", "SUN");
				String proto = "TLS";
				SSLContext con = SSLContext.getInstance(proto);
	
				FileInputStream fisKeystore = new FileInputStream(mConfReader.getKeyStoreFileName());
	
				// make an instance of the keystore
				logger.info("Keystore type: " + KeyStore.getDefaultType());
	
				KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
	
				// Now load the keystore. I have yet to find a parameter other
				// thatn null that works in the second parameter
				ks.load(fisKeystore, null);
	
				// Key manager algorithm this is currently the only default
				// algorithn
				// that works with JSSE
				String keyManageAlgo = "SunX509";
	
				// Get the key manager factory as we have the preliminaries out of
				// the way.
				KeyManagerFactory km = KeyManagerFactory.getInstance(keyManageAlgo);
				// intialize the key manager factory
				km.init(ks, mConfReader.getKeyStorePasswort().toCharArray());
	
				// Now get the key managers
				KeyManager[] keymanage = km.getKeyManagers();
				for (int i = 0; i < keymanage.length; i++)
					logger.trace("keymanager number " + i + ": " + keymanage[i]);
	
				FileInputStream fisTruststore = new FileInputStream(mConfReader.getTrustStoreFileName());
				KeyStore ts = KeyStore.getInstance(KeyStore.getDefaultType());
				ts.load(fisTruststore, null);
	
				// Now get the Trust Manager stuff
				TrustManagerFactory tmFactory = TrustManagerFactory
						.getInstance(keyManageAlgo);
				tmFactory.init(ts);
				TrustManager[] tmArray = tmFactory.getTrustManagers();
				for (int i = 0; i < tmArray.length; i++){
					logger.trace("trustmanager number " + i + ": " + tmArray[i]);
				}
	
				// Now intialize the keymanagers
				con.init(keymanage, tmArray, random);
	
				// finally we can create a socket factory
				SSLServerSocketFactory sf = con.getServerSocketFactory();			
				
				mCredentialAuthSocket = (SSLServerSocket)sf.createServerSocket(credentialAuthPort);
				mCredentialAuthSocket.setWantClientAuth(true);
				mCredentialAuthSocket.setNeedClientAuth(true);
				
				// yes! ugly!
				// if one of those guys happen we have to abort the whole thing
				// anyway, so throw up a ServerInitialException, we still output
				// the stack trace for convenience.
			} catch (IOException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			} catch (KeyManagementException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			} catch (KeyStoreException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			} catch (NoSuchAlgorithmException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			} catch (CertificateException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			} catch (UnrecoverableKeyException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			} catch (NoSuchProviderException e) {
				e.printStackTrace();
				throw new ServerInitialException(e.getMessage());
			}
		}
		
		if (mBasicAuthSocket == null && mCredentialAuthSocket == null) {
			throw new ServerInitialException(
					"No Ports in property file specified. Expected basic and/or"
							+ " credential authentication");
		}				
}

	private ChannelIdentifier getChannelIdentifier(Socket s){
		return new ChannelIdentifier(s.getInetAddress().getHostAddress(), s.getPort(), 
				++mCounter);
	}
	
	@Override
	public void run() {
		if (mBasicAuthSocket != null) {
			Thread basic = new Thread(new Runnable() {				
				@Override
				public void run() {
					logger.info("Listening on port " + mBasicAuthSocket.getLocalPort() 
							+ " for basic access authentication");
					while (true) {
						Socket basicSocket;						
						ChannelThread ct;
						ChannelIdentifier ci;
						mBasicAuth = new BasicAuth(mBasicAuthProvider);
						try {
							basicSocket = mBasicAuthSocket.accept();							
							ci = getChannelIdentifier(basicSocket);							
							ct = new ChannelThread(basicSocket, mBasicAuth, ci,
									mEventQueue, mCounter);
							mChannelRep.add(ct);
							new Thread(ct).start();														
						} catch (IOException e) {
							logger.info("IOException in basic authentication");						
						}				
					}
				}
			});
			basic.start();		
		}

		if (mCredentialAuthSocket != null) {
			Thread credential = new Thread(new Runnable() {

				@Override
				public void run() {
					logger.info("Listening on port " + mCredentialAuthSocket.getLocalPort() 
							+ " for certificate based authentication");
					while (true) {
						Socket credentialSocket;
						ChannelThread ct;
						ChannelIdentifier ci;						
						try {
							credentialSocket = mCredentialAuthSocket.accept();
							mCredentialAuth = new CertificateAuth(credentialSocket);							
							ci = getChannelIdentifier(credentialSocket);	
							ct = new ChannelThread(credentialSocket, mCredentialAuth, ci, 
									mEventQueue, mCounter);
							mChannelRep.add(ct);
							new Thread(ct).start();							
						} 
						catch (IOException e) {
							logger.info("IOException in certificate based authentication");		
						}				
					}
				}
			});
			credential.start();
	}
 }	
}