/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel.graph
 * File:    AccessRequest.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel.graph;

import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.IdentifierConstructionException;

/**
 * AccessRequest class
 * 
 * Implementation of a AccessRequest Identifier
 * 
 * @author aw
 * @version 0.1
 *
 * created: 26.11.09
 * changes: 
 *	26.11.09 aw - Created first version of this class
 *	27.11.09 aw - equals() check for null, constructor with default
 *  	administrative-domain
 *	28.11.09 aw - equals() use super.equals() to compare administrative
 *  	domains.
 *	09.12.09 aw - implemented copy()
 *	17.12.09 aw - added type constructor
 *  04.02.10 aw - toString()
 *  07.02.10 aw - modified toString()
 */
public class AccessRequest extends IdentifierWithAdministrativeDomain {
	
	private final String name;
	private final int hashCode;
	
	/**
	 * Create a AccessRequest Object based on
	 * a administrative-domain and a name.
	 * 
	 * @param ad
	 * @param name
	 * @throws IdentifierConstructionException 
	 */
	public AccessRequest(final String name, final String ad) throws IdentifierConstructionException {
		super(IdentifierType.instanceOfTypeWith("access-request"), ad);
		if (name == null || name.length() == 0) {
			throw new IdentifierConstructionException("AccessRequest: " +
					"name is invalid");
		}
		this.name = name;
		hashCode = (super.hashCode() + name).hashCode();
	}
	
	/**
	 * Create a AccessRequest only based on a given name.
	 * The administrative-domain will be set to ""
	 * 
	 * @param name
	 * @throws IdentifierConstructionException
	 */
	public AccessRequest(final String name) throws IdentifierConstructionException {
		this(name, "");
	}
	
	public String getName() {
		return name;
	}
	
	/**
	 * Two AccessRequests are the same if
	 *   - they have the same reference
	 *   - they have the same fields
	 *   
	 */
	@Override
	public boolean equals(Object o) {
		AccessRequest ar;
		if (this == o)
			return true;
		
		if (super.equals(o)) {
			if (o instanceof AccessRequest) {
				ar = (AccessRequest)o;
				return name.equals(ar.getName());
			}
		}
		return false;
	}
		
	
	
	public int hashCode() {
		return hashCode;
	}

	/* (non-Javadoc)
	 * @see de.fhhannover.inform.iron.mapserver.datamodel.service.data.graph.Identifier#copy()
	 */
	public Identifier copy() {
		AccessRequest ar = null;
		try {
			
			ar = new AccessRequest(this.getName(), this.getAdministrativeDomain());
		} catch (IdentifierConstructionException e) {
			e.printStackTrace();						/* should not fail */
		}
		return ar;
	}
	
	public String toString() {
		final String ad = getAdministrativeDomain();
		final StringBuffer sb = new StringBuffer();
		sb.append("Access Request<name=" + name);
		if (!(ad == null || ad.length() == 0)) {
			sb.append(" ad=" + ad);
		}
		sb.append(">");
		return sb.toString();
	}
}
 
